from typing import Union, BinaryIO

from loko_client.business.base_client import OrchestratorClient, AsyncOrchestratorClient


class FSClient(OrchestratorClient):
    """
        Orchestrator client used to access to Loko data.

        Args:
            gateway (str): Gateway URL.
    """

    def ls(self, path: str):
        """
            List directory contents.

            Parameters:
                path (str): The path to the parent folder.

            Returns:
                List[dict]: List of contents.

        """
        r = self.u.files[path].get()
        return r.json()['items']

    def read(self, path: str, mode='rb'):
        """
            Read file.

            Parameters:
                path (str): The path to the file.
                mode (str): The mode in which the file is read. Available modes are: "r" and "rb". Default: 'rb'

            Returns:
                 Union[str, bytes]: File content.
        """
        r = self.u.files[path].get()
        if mode=='rb':
            return r.content
        return r.content.decode()

    def update(self, path: str, body: Union[bytes, BinaryIO]=None):
        """
            Update file.

            Parameters:
                path (str): The path to the file.
                body (Union[bytes, BinaryIO]): File's content.

        """
        r = self.u.files[path].post(data=body)
        return r.text

    def save(self, path: str, body: Union[bytes, BinaryIO]=None):
        """
            Save file or directory.

            Parameters:
                path (str): The path to the file.
                body (Union[bytes, BinaryIO]): File's content. `None` if you want to save a directory. Default: None

        """
        if body:
            r = self.u.files[path].post(data=body)
        else:
            r = self.u.files[path].post()
        return r.text

    def delete(self, path: str):
        """
            Delete file or directory.

            Parameters:
                path (str): The path to the file or directory.

        """
        r = self.u.files[path].delete()
        return r.text

    def copy(self, path: str, new_path: str):
        """
            Copy file or directory.

            Parameters:
                path (str): The path to the file or directory.
                new_path (str): The new path to the file or directory.

        """
        r = self.u.copy[path].post(json=dict(path=new_path))
        return r.text

    def move(self, path: str, new_path: str):
        """
            Move file or directory.

            Parameters:
                path (str): The path to the file or directory.
                new_path (str): The new path to the file or directory.

        """
        r = self.u.files[path].patch(json=dict(path=new_path))
        return r.text

class AsyncFSClient(AsyncOrchestratorClient):
    """
        Asynchronous Orchestrator client used to access to Loko data.

        Args:
            gateway (str): Gateway URL.
            timeout (float): The maximal number of seconds for the whole operation including connection establishment,
                request sending and response reading. Default: 300
    """

    def __init__(self, gateway, timeout=None):
        super().__init__(gateway, timeout)

    async def ls(self, path: str):
        """
            List directory contents.

            Parameters:
                path (str): The path to the parent folder.

            Returns:
                List[dict]: List of contents.

        """
        resp = await self.u.files[path].request('GET')
        r = await resp.json()
        return r['items']

    async def read(self, path: str, mode='rb', content=False):
        """
            Read file.

            Parameters:
                path (str): The path to the file.
                mode (str): The mode in which the file is read. Available modes are: "r" and "rb". Default: 'rb'
                content (bool): Set to `True` to return the response content. Default: False

            Returns:
                 Union[str, bytes, aiohttp.streams.StreamReader]: File content.
        """
        resp = await self.u.files[path].request('GET')
        _content = resp.content
        if content:
            return _content
        _content = await _content.read()
        if mode=='rb':
            return _content
        return _content.decode()

    async def update(self, path: str, body: Union[bytes, BinaryIO]=None):
        """
            Update file.

            Parameters:
                path (str): The path to the file.
                body (Union[bytes, BinaryIO]): File's content.

        """
        r = await self.u.files[path].request('POST', data=body)
        return await r.text()

    async def save(self, path: str, body: Union[bytes, BinaryIO]=None):
        """
            Save file or directory.

            Parameters:
                path (str): The path to the file.
                body (Union[bytes, BinaryIO]): File's content. `None` if you want to save a directory. Default: None

        """
        if body:
            r = await self.u.files[path].request('POST', data=body)
        else:
            r = await self.u.files[path].request('POST')
        return await r.text()

    async def delete(self, path: str):
        """
            Delete file or directory.

            Parameters:
                path (str): The path to the file or directory.

        """
        r = await self.u.files[path].request('DELETE')
        return await r.text()

    async def copy(self, path: str, new_path: str):
        """
            Copy file or directory.

            Parameters:
                path (str): The path to the file or directory.
                new_path (str): The new path to the file or directory.

        """
        r = await self.u.copy[path].request('POST', json=dict(path=new_path))
        return await r.text()

    async def move(self, path: str, new_path: str):
        """
            Move file or directory.

            Parameters:
                path (str): The path to the file or directory.
                new_path (str): The new path to the file or directory.

        """
        r = await self.u.files[path].request('PATCH', json=dict(path=new_path))
        return await r.text()



from loko_client.utils.gobjects_utils import GObject


class Nodes:
    """
        Nodes contained by a Loko tab.

        Args:
            nodes (dict): Dictionary containing the nodes ids as keys. Each node is a
                :py:meth:`~loko_client.utils.gobjects_utils.GObject`.
    """
    def __init__(self, nodes):
        self.nodes = {node['id']: GObject('Node', **node) for node in nodes}

    def all(self):
        """ Return all nodes' ids. """
        return list(self.nodes.keys())

    def search(self, key, value):
        """ Search for specific nodes using a condition.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> ### get all nodes contained by the project first_project and tab main
                >>> nodes = pclient.get_project('first_project').tabs.main.nodes
                >>> ### get all 'HTTP REQUEST' nodes
                >>> print(list(nodes.search('data.name', 'HTTP Request')))

            Args:
                key (str): Node's attribute. Use dot notation to concatenate nested keys.
                value: Value that the node's attribute must assume.
        """
        for _id,node in self.nodes.items():
            val = node
            for k in key.split('.'):
                val = val.get(k)
                if not val:
                    break
            if val==value:
                yield _id

    def __getitem__(self, item):
        return self.nodes[item]


class Edges:
    """
        Edges contained by a Loko tab.

        Args:
            edges (dict): Dictionary containing the edges ids as keys. Each edge is a
                :py:meth:`~loko_client.utils.gobjects_utils.GObject`.
        """
    def __init__(self, edges):
        self.edges = {edge['id']: GObject('Edge', **edge) for edge in edges}

    def all(self):
        """ Return all edges' ids. """
        return list(self.edges.keys())


class Project:
    """
        A Loko project.

        Attributes:
            name (str): The project name.
            id (str): The id of the project.
            description (str): The project description.
            created_on (str): The creation date: ``%d/%m/%Y, %H:%M:%S``.
            last_modify (str): The last modify date: ``%d/%m/%Y, %H:%M:%S``.
            graphs (dict): Tabs flows.
            open (List[str]): The opened tabs.
            active (str): The name of the active tab.
            version (str): Loko version.
            deployed (bool): `True` if the project is deployed.
            tabs (GObject): Object containing the name of the project's tabs as attributes. Each tab is a
                :py:meth:`~loko_client.utils.gobjects_utils.GObject` containing
                :py:meth:`~loko_client.model.projects.Nodes` and
                :py:meth:`~loko_client.model.projects.Edges` information.
            """
    def __init__(self, name, id, description, created_on, last_modify, graphs, open, active, version, deployed=None,
                 **kwargs):
        self.name = name
        self.id = id
        self.description = description
        self.created_on = created_on
        self.last_modify = last_modify
        self.open = open
        self.active = active
        self.version = version
        self.deployed = deployed

        self.graphs = graphs

        self.tabs = GObject('Tabs', **{name: GObject('Tab', nodes=Nodes(tab['nodes']), edges=Edges(tab['edges']))
                                       for name,tab in self.graphs.items()})







from loko_client.business.fs_client import FSClient

GATEWAY = 'http://localhost:9999/routes/'

fsclient = FSClient(gateway=GATEWAY)
print(fsclient.ls('data/data/datasets'))
content = fsclient.read('data/data/datasets/titanic.csv', mode='rb')
print(content.decode())
### save file ###
print(fsclient.save('data/data/test/test.csv', content))
print(fsclient.save('data/data/test2/test.csv', content))
### save dir ###
print(fsclient.save('data/data/test3'))
### delete dir ###
print(fsclient.delete('data/data/test2/'))
### delete file ###
print(fsclient.delete('data/data/test/test.csv'))
### copy file in existing directory ###
print(fsclient.save('data/data/test3/test.csv', content))
print(fsclient.copy('data/data/test3/test.csv', 'data/data/test3/test2.csv'))
### move file in existing directory ###
print(fsclient.move('data/data/test3/test.csv', 'data/data/test.csv'))
### update ###
print(fsclient.update('data/data/test.csv', 'hello'.encode()))



import os

import requests
from aiohttp import ClientSession, ClientTimeout


class URLRequest:
    """
        Synchronous HTTP requests handler based on requests.

        Args:
            url (str): Base URL for the requests.
    """
    def __init__(self, url):
        self.url = url

    def get(self, **kwargs):
        """ Sends a GET request. """
        return requests.get(self.url, **kwargs)

    def post(self, **kwargs):
        """ Sends a POST request. """
        return requests.post(self.url, **kwargs)

    def delete(self, **kwargs):
        """ Sends a DELETE request. """
        return requests.delete(self.url, **kwargs)

    def put(self, **kwargs):
        """ Sends a PUT request. """
        return requests.put(self.url, **kwargs)

    def patch(self, **kwargs):
        """ Sends a PATCH request. """
        return requests.patch(self.url, **kwargs)

    def __getattr__(self, k):
        return URLRequest(os.path.join(self.url, k))

    def __getitem__(self, k):
        return URLRequest(os.path.join(self.url, k))

class AsyncURLRequest:
    """
        Synchronous HTTP requests handler based on aiohttp.

        Args:
            url (str): Base URL for the requests.
            timeout (float): The maximal number of seconds for the whole operation including connection establishment,
                request sending and response reading. Default: 300
    """
    def __init__(self, url, timeout=None):
        self.url = url
        self.timeout = timeout
    async def request(self, method='GET', **kwargs):
        """
            Sends a request.

            Args:
                method (str): Request method: ``GET``, ``OPTIONS``, ``HEAD``, ``POST``, ``PUT``, ``PATCH``,
                    or ``DELETE``.
        """
        async with ClientSession(timeout=ClientTimeout(total=self.timeout)) as session:
            return await session.request(url=self.url, method=method, **kwargs)

    def __getattr__(self, k):
        return AsyncURLRequest(os.path.join(self.url, k))

    def __getitem__(self, k):
        return AsyncURLRequest(os.path.join(self.url, k))



import time

from loko_client.business.projects_client import AsyncProjectsClient, ProjectsClient
import asyncio

GATEWAY = 'http://localhost:9999/routes/'

async_client = False
pclient = AsyncProjectsClient(gateway=GATEWAY) if async_client else ProjectsClient(gateway=GATEWAY)

async def test(pclient):

    print(pclient)

    ### PROJECTS ###
    if async_client:
        print(f'ALL PROJECTS: {await pclient.all_projects()}')
    else:
        print(f'ALL PROJECTS: {pclient.all_projects()}')
    if async_client:
        prj = await pclient.get_project("hello")
    else:
        prj = pclient.get_project("hello")
    print(f'ALL TABS: {prj.tabs.all()}')
    all_nodes = []
    for tab in prj.tabs.all():
        all_nodes += prj.tabs[tab].nodes.nodes.values()
    print(f'ALL NODES: {all_nodes}')
    print(f'ALL NODES IDS: {[node.id for node in all_nodes]}')

    nodes = prj.tabs.main.nodes
    print(f'ALL MAIN NODES IDS: {nodes.all()}')
    print(f'ALL MAIN NODES: {nodes.nodes}')

    res = prj.tabs.main.nodes.search('data.name', 'HTTP Request')
    print(f'SEARCH FOR HTTP Request: {list(res)}')
    for node_id in res:
        node = prj.tabs.main.nodes[node_id]
        print(node.to_dict())
        print(node.data['name'])

    res = prj.tabs.main.nodes.search('data.options.values.alias', 'prova')
    print(f'SEARCH FOR alias=prova: {list(res)}')
    for node_id in res:
        node = prj.tabs.main.nodes[node_id]
        print(node.to_dict())
        print(node.data['options']['values']['alias'])

    ### TASKS ###
    if async_client:
        running = await pclient.running_tasks()
    else:
        running = pclient.running_tasks()
    print(f'RUNNING: {running}')
    for task in running:
        print(task)
        print(f'project: {task["project"]} - tab: {task["graph"]} - node_name: {task["source"]} - '
              f'uid: {task["uid"]} - user: {task["user"]} - started: {task["startedAt"]}')
    if running:
        print(f'CANCEL TASK: project {running[0]["project"]} - tab {running[0]["graph"]} - uid {running[0]["uid"]}')
        if async_client:
            print(await pclient.cancel_task(uid=running[0]['uid']))
            print(f'RUNNING: {await pclient.running_tasks()}')
        else:
            print(pclient.cancel_task(uid=running[0]['uid']))
            print(f'RUNNING: {pclient.running_tasks()}')
    time.sleep(1)
    if async_client:
        trigger_id = list((await pclient.get_project('hello')).tabs.main.nodes.search('data.name', 'Trigger'))[0]
        await pclient.trigger('hello', trigger_id)
    else:
        trigger_id = list(pclient.get_project('hello').tabs.main.nodes.search('data.name', 'Trigger'))[0]
        pclient.trigger('hello', trigger_id)
    if async_client:
        print(f'RUNNING: {await pclient.running_tasks()}')
        print(f'DEPLOYMENT STATUS {await pclient.deployment_status("hello")}')
        print(await pclient.undeploy('hello'))
        print(f'DEPLOYMENT STATUS {await pclient.deployment_status("hello")}')
        print(await pclient.deploy('hello'))
        print(f'DEPLOYMENT STATUS {await pclient.deployment_status("hello")}')
    else:
        print(f'RUNNING: {pclient.running_tasks()}')
        print(f'DEPLOYMENT STATUS {pclient.deployment_status("hello")}')
        print(pclient.undeploy('hello'))
        print(f'DEPLOYMENT STATUS {pclient.deployment_status("hello")}')
        print(pclient.deploy('hello'))
        print(f'DEPLOYMENT STATUS {pclient.deployment_status("hello")}')


loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)
task = loop.create_task(test(pclient))
r = loop.run_until_complete(task)


from loko_client.business.base_client import OrchestratorClient, AsyncOrchestratorClient
from loko_client.model.projects import Project


class ProjectsClient(OrchestratorClient):
    """
        Orchestrator client used to manage Loko projects.

        Args:
            gateway (str): Gateway URL.
    """

    def all_projects(self):
        """
            Return Loko projects.

        """
        r = self.u.projects.get()
        return r.json()

    def get_project(self, id: str):
        """
            Return Loko project.

            Args:
                id (str): Loko project id.

            Returns:
                Project: Loko project.

        """
        r = self.u.projects[id].get()
        return Project(**r.json())

    def running_tasks(self):
        """
            Return Loko running tasks.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> running = pclient.running_tasks()
                >>> for task in running:
                >>>     print(f'project: {task["project"]} - tab: {task["graph"]} - node_name: {task["source"]} - '
                >>>     f'uid: {task["uid"]} - user: {task["user"]} - started: {task["startedAt"]}')

            Returns:
                List[dict]: List of dictionaries containing information of running tasks.
        """
        r = self.u.tasks.get()
        return r.json()

    def cancel_all_tasks(self):
        """ Cancel all Loko running tasks. """
        r = self.u.tasks.delete()
        return r.json()

    def cancel_task(self, uid: str):
        """
            Cancel a single Loko task using its uid.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> running = pclient.running_tasks()
                >>> print(f'RUNNING: {running}')
                >>> if running:
                >>>     print(f'CANCEL TASK: project {running[0]["project"]} - tab {running[0]["graph"]} - uid {running[0]["uid"]}')
                >>>     print(pclient.cancel_task(uid=running[0]['uid']))
                >>>     print(f'RUNNING: {pclient.running_tasks()}')

            Args:
                uid (str): Loko task uid.
        """
        r = self.u.tasks[uid].delete()
        return r.json()

    def trigger(self, project_id: str, component_id: str):
        """
            Start Loko task using *project_id* and *component_id*.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> component_id = list(pclient.get_project('first_project').tabs.main.nodes.search('data.options.values.alias',
                >>>                                                                                 'My block'))[0]
                >>> pclient.trigger('first_project', component_id)
                >>> print(pclient.running_tasks())

            Args:
                project_id (str): Loko project id.
                component_id (str): Loko node id.

        """
        r = self.u.projects[project_id].trigger.post(json=dict(id=component_id))
        return r.json()

    def deploy(self, project_id: str):
        """
            Deploy Loko project.

            Args:
                project_id (str): Loko project id.
        """
        r = self.u.deploy[project_id].get()
        return r.json()

    def undeploy(self, project_id: str):
        """
            Undeploy Loko project.

            Args:
                project_id (str): Loko project id.
        """
        r = self.u.undeploy[project_id].get()
        return r.json()

    def deployment_status(self, project_id: str):
        """
            Check Loko project's deployment status.

            Args:
                project_id (str): Loko project id.

            Returns:
                dict: Dictionary containing project `status` and associated `guis`.
        """
        r = self.u.deployment_status[project_id].get()
        return r.json()


class AsyncProjectsClient(AsyncOrchestratorClient):
    """
        Orchestrator client used to manage Loko projects.

        Args:
            gateway (str): Gateway URL.
            timeout (float): The maximal number of seconds for the whole operation including connection establishment,
                request sending and response reading. Default: 300
    """

    async def all_projects(self):
        """
            Return Loko projects.

        """
        r = await self.u.projects.request('GET')
        return await r.json()

    async def get_project(self, id: str):
        """
            Return Loko project.

            Args:
                id (str): Loko project id.

            Returns:
                Project: Loko project.

        """
        r = await self.u.projects[id].request('GET')
        return Project(** await r.json())

    async def running_tasks(self):
        """
            Return Loko running tasks.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> running = await pclient.running_tasks()
                >>> for task in running:
                >>>     print(f'project: {task["project"]} - tab: {task["graph"]} - node_name: {task["source"]} - '
                >>>     f'uid: {task["uid"]} - user: {task["user"]} - started: {task["startedAt"]}')

            Returns:
                List[dict]: List of dictionaries containing information of running tasks.
        """
        r = await self.u.tasks.request('GET')
        return await r.json()

    async def cancel_all_tasks(self):
        """ Cancel all Loko running tasks. """
        r = await self.u.tasks.request('DELETE')
        return await r.json()

    async def cancel_task(self, uid: str):
        """
            Cancel a single Loko task using its uid.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> running = await pclient.running_tasks()
                >>> print(f'RUNNING: {running}')
                >>> if running:
                >>>     print(f'CANCEL TASK: project {running[0]["project"]} - tab {running[0]["graph"]} - uid {running[0]["uid"]}')
                >>>     print(await pclient.cancel_task(uid=running[0]['uid']))
                >>>     print(f'RUNNING: {await pclient.running_tasks()}')

            Args:
                uid (str): Loko task uid.
        """
        r = await self.u.tasks[uid].request('DELETE')
        return await r.json()

    async def trigger(self, project_id: str, component_id: str):
        """
            Start Loko task using *project_id* and *component_id*.

            Example:
                >>> GATEWAY = 'http://localhost:9999/routes/'
                >>> pclient = ProjectsClient(gateway=GATEWAY)
                >>> prj = await pclient.get_project("first_project")
                >>> component_id = list(prj.tabs.main.nodes.search('data.options.values.alias',
                >>>                                                'My block'))[0]
                >>> pclient.trigger('first_project', component_id)
                >>> print(pclient.running_tasks())

            Args:
                project_id (str): Loko project id.
                component_id (str): Loko node id.

        """
        r = await self.u.projects[project_id].trigger.request('POST', json=dict(id=component_id))
        return await r.json()

    async def deploy(self, project_id: str):
        """
            Deploy Loko project.

            Args:
                project_id (str): Loko project id.
        """
        r = await self.u.deploy[project_id].request('GET')
        return await r.json()

    async def undeploy(self, project_id: str):
        """
            Undeploy Loko project.

            Args:
                project_id (str): Loko project id.
        """
        r = await self.u.undeploy[project_id].request('GET')
        return await r.json()

    async def deployment_status(self, project_id: str):
        """
            Check Loko project's deployment status.

            Args:
                project_id (str): Loko project id.

            Returns:
                dict: Dictionary containing project `status` and associated `guis`.
        """
        r = await self.u.deployment_status[project_id].request('GET')
        return await r.json()



import sys
import asyncio
from loko_client.business.fs_client import AsyncFSClient

async def get_chunked_file(path):
    resp = await fsclient.read(path, content=True)
    async for line in resp:
        print(line)

GATEWAY = 'http://localhost:9999/routes/'
fsclient = AsyncFSClient(gateway=GATEWAY)

loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)
task = loop.create_task(fsclient.ls('data/data/datasets'))
r = loop.run_until_complete(task)
print(r)
### read file ###
task = loop.create_task(fsclient.read('data/data/datasets/titanic.csv', mode='r'))
content = loop.run_until_complete(task)
print(content)
### read content ###
task = loop.create_task(get_chunked_file('data/data/datasets/titanic.csv'))
loop.run_until_complete(task)
# sys.exit(0)
### save file ###
task = loop.create_task(fsclient.save('data/data/test/test.csv', body=content))
r = loop.run_until_complete(task)
print(r)
task = loop.create_task(fsclient.save('data/data/test2/test.csv', body=content))
r = loop.run_until_complete(task)
print(r)
### save dir ###
task = loop.create_task(fsclient.save('data/data/test3'))
r = loop.run_until_complete(task)
print(r)
### delete dir ###
task = loop.create_task(fsclient.delete('data/data/test2/'))
r = loop.run_until_complete(task)
print(r)
### delete file ###
task = loop.create_task(fsclient.delete('data/data/test/test.csv'))
r = loop.run_until_complete(task)
print(r)
### copy file in existing directory ###
task = loop.create_task(fsclient.save('data/data/test3/test.csv', content))
r = loop.run_until_complete(task)
print(r)
task = loop.create_task(fsclient.copy('data/data/test3/test.csv', 'data/data/test3/test2.csv'))
r = loop.run_until_complete(task)
print(r)
### move file in existing directory ###
task = loop.create_task(fsclient.move('data/data/test3/test.csv', 'data/data/test.csv'))
r = loop.run_until_complete(task)
print(r)
### update ###
task = loop.create_task(fsclient.update('data/data/test.csv', 'hello'.encode()))
r = loop.run_until_complete(task)
print(r)


import setuptools,os

def get_requirements():
    return [x for x in open("./requirements.txt").read().split("\n") if not x.startswith("pkg_resources")]

def get_long_description() -> str:

    readme_filepath = os.path.join(os.path.dirname(__file__), "README.md")
    with open(readme_filepath) as f:
        return f.read()

import json
ppom=json.load(open("./ppom.json"))
version=ppom['version']

setuptools.setup(
    name="loko-client",
    version=version,
    author="Live Tech",
    author_email="f.dantonio@ilivetech.it",
    description="A client for Loko orchestrator services",
    long_description=get_long_description(),
    python_requires='>3.5.0',
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=get_requirements()

)


from abc import ABC

from loko_client.utils.requests_utils import URLRequest, AsyncURLRequest


GATEWAY = 'http://localhost:9999/routes/'

class OrchestratorClient(ABC):
    """
        An abstract base orchestrator client
    """

    def __init__(self, gateway=GATEWAY):
        self.u = URLRequest(gateway).orchestrator

class AsyncOrchestratorClient(ABC):
    """
        An abstract base orchestrator client
    """

    def __init__(self, gateway, timeout=None):
        self.u = AsyncURLRequest(gateway, timeout).orchestrator




class GObject(object):
    """
        A generic object.

        Args:
            class_name (str): Object name.
            kwargs: All kwargs represents attributes/items of the class.
    """
    def __init__(self, class_name='GenericObj', **kwargs):
        self.class_name = class_name
        self.kwargs = kwargs

    def all(self):
        """ Return all attributes/items names. """
        return list(self.kwargs.keys())

    def __repr__(self):
        return f"<class '{self.class_name}'>"

    def __getattr__(self, item):
        return self.kwargs[item]

    def __getitem__(self, item):
        return self.kwargs[item]

    def get(self, item, default=None):
        """ Return the value for key if key is in the items, else default. """
        return self.kwargs.get(item, default)

    def to_dict(self):
        """ Return kwargs dictionary. """
        return self.kwargs
